@@ -51,6 +51,7 @@ BEGIN {
 
 use strict;
 use CPANPLUS::Configure;
+use CPANPLUS::Error ();
 
 use File::Path      qw[rmtree];
 use FileHandle;
@@ -66,6 +67,7 @@ use constant TEST_CONF_PREREQ           => 'Cwd';
 use constant TEST_CONF_MODULE           => 'Foo::Bar::EU::NOXS';
 use constant TEST_CONF_INST_MODULE      => 'Foo::Bar';
 use constant TEST_CONF_INVALID_MODULE   => 'fnurk';
+use constant TEST_CONF_MIRROR_DIR       => 'dummy-localmirror';
 
 ### we might need this Some Day when we're installing into
 ### our own sandbox. see t/20.t for details
@@ -112,30 +114,81 @@ sub gimme_conf {
     $conf->set_conf( dist_type  => '' );
     $conf->set_conf( signature  => 0 );
 
+    ### dmq tells us that we should run with /nologo
+    ### if using nmake, as it's very noise otherwise.
+    {   my $make = $conf->get_program('make');
+        if( $make and basename($make) =~ /^nmake/i and
+            $make !~ m|/nologo|
+        ) {
+            $make .= ' /nologo';
+            $conf->set_program( make => $make );
+        }
+    }
+    
     _clean_test_dir( [
         $conf->get_conf('base'),     
+        TEST_CONF_MIRROR_DIR,
 #         TEST_INSTALL_DIR_LIB,
 #         TEST_INSTALL_DIR_BIN,
 #         TEST_INSTALL_DIR_MAN1, 
 #         TEST_INSTALL_DIR_MAN3,
-    ], 1 );
+    ], (  $ENV{PERL_CORE} ? 0 : 1 ) );
         
     return $conf;
 };
 
-my $fh;
-my $file = ".".basename($0).".output";
-sub output_handle {
-    return $fh if $fh;
+{
+    my $fh;
+    my $file = ".".basename($0).".output";
+    sub output_handle {
+        return $fh if $fh;
+        
+        $fh = FileHandle->new(">$file")
+                    or warn "Could not open output file '$file': $!";
+       
+        $fh->autoflush(1);
+        return $fh;
+    }
     
-    $fh = FileHandle->new(">$file")
-                or warn "Could not open output file '$file': $!";
-   
-    $fh->autoflush(1);
-    return $fh;
+    sub output_file { return $file }
+    
+    
+    my $env = 'PERL5_CPANPLUS_TEST_VERBOSE';
+    ### redirect output from msg() and error() output to file
+    unless( $ENV{$env} ) {
+    
+        print "# To run tests in verbose mode, set ".
+              "\$ENV{PERL5_CPANPLUS_TEST_VERBOSE} = 1\n" unless $ENV{PERL_CORE};
+    
+        unlink $file;   # just in case
+    
+        $CPANPLUS::Error::ERROR_FH  =
+        $CPANPLUS::Error::ERROR_FH  = output_handle();
+        
+        $CPANPLUS::Error::MSG_FH    =
+        $CPANPLUS::Error::MSG_FH    = output_handle();
+        
+    }        
 }
 
-sub output_file { return $file }
+
+### clean these files if we're under perl core
+END { 
+    if ( $ENV{PERL_CORE} ) {
+        close output_handle(); 1 while unlink output_file();
+
+        _clean_test_dir( [
+            gimme_conf->get_conf('base'),   
+            TEST_CONF_MIRROR_DIR,
+    #         TEST_INSTALL_DIR_LIB,
+    #         TEST_INSTALL_DIR_BIN,
+    #         TEST_INSTALL_DIR_MAN1, 
+    #         TEST_INSTALL_DIR_MAN3,
+        ], 0 ); # DO NOT be verbose under perl core -- makes tests fail
+    }
+}
+
+
 
 ### whenever we start a new script, we want to clean out our
 ### old files from the test '.cpanplus' dir..
@@ -145,6 +198,9 @@ sub _clean_test_dir {
 
     for my $dir ( @$dirs ) {
 
+        ### no point if it doesn't exist;
+        next unless -d $dir;
+
         my $dh;
         opendir $dh, $dir or die "Could not open basedir '$dir': $!";
         while( my $file = readdir $dh ) { 
@@ -154,13 +210,13 @@ sub _clean_test_dir {
             
             ### directory, rmtree it
             if( -d $path ) {
-                print "Deleting directory '$path'\n" if $verbose;
+                print "# Deleting directory '$path'\n" if $verbose;
                 eval { rmtree( $path ) };
                 warn "Could not delete '$path' while cleaning up '$dir'" if $@;
            
             ### regular file
             } else {
-                print "Deleting file '$path'\n" if $verbose;
+                print "# Deleting file '$path'\n" if $verbose;
                 1 while unlink $path;
             }            
         }       
